<?
class DH_ConvDB {

	const TYPE_SHOT = 'shot';
	const TYPE_FLV = 'flv';


	static function touch($drop_jobs = false, $drop_postproc = false) {
		global $gCfg;
		$db = DB::get();
		$aDb = DB::get(null, 'animebase');

		if ($drop_jobs) {
			$db->e("DROP TABLE IF EXISTS {$gCfg['storage']['table_paths']}");
			$db->e("DROP TABLE IF EXISTS {$gCfg['storage']['table_jobs']}");
			$db->e("DROP TABLE IF EXISTS {$gCfg['storage']['table_log']}");
		}
		if ($drop_postproc) $aDb->e("DROP TABLE IF EXISTS {$gCfg['storage']['table_postproc']}");

		$db->e("
			CREATE TABLE IF NOT EXISTS {$gCfg['storage']['table_paths']} (
				id INT(10) UNSIGNED NOT NULL auto_increment PRIMARY KEY,
				path VARCHAR(255) NOT NULL UNIQUE,
				timestamp INT(10) UNSIGNED NULL
			) ENGINE=MyISAM DEFAULT CHARSET=utf8;
		");
		$db->e("
			CREATE TABLE IF NOT EXISTS {$gCfg['storage']['table_jobs']} (
				id INT(10) UNSIGNED NOT NULL auto_increment PRIMARY KEY,
				path_id INT(10) UNSIGNED NOT NULL,
				video VARCHAR(255) NULL UNIQUE,
				sub VARCHAR(255) NULL UNIQUE,
				status TINYINT UNSIGNED NOT NULL DEFAULT 0,
				timestamp INT(10) UNSIGNED NULL
			) ENGINE=MyISAM DEFAULT CHARSET=utf8;
		");
		$db->e("
			CREATE TABLE IF NOT EXISTS {$gCfg['storage']['table_log']} (
				id INT(10) UNSIGNED NOT NULL auto_increment PRIMARY KEY,
				job_id INT(10) UNSIGNED NOT NULL,
				message TEXT NULL,
				timestamp INT(10) UNSIGNED NULL
			) ENGINE=MyISAM DEFAULT CHARSET=utf8;
		");

		$aDb->e("
			CREATE TABLE IF NOT EXISTS {$gCfg['storage']['table_postproc']} (
				flv VARCHAR(255) NULL PRIMARY KEY,
				shot VARCHAR(255) NULL,
				video_id VARCHAR(255) NOT NULL,
				timestamp INT(10) UNSIGNED NULL,
				processed TINYINT UNSIGNED NOT NULL DEFAULT 0
			) ENGINE=MyISAM DEFAULT CHARSET=utf8;
		");
	}


	static function path($path) {
		global $gCfg;
		$db = DB::get();
		return $db->val($gCfg['storage']['table_paths'], 'id', compact('path'));
	}


	static function add_path($path, $entries) {
		global $gCfg;
		$db = DB::get();
		$timestamp = Time::now();
		$path_id = $db->ins($gCfg['storage']['table_paths'], compact('path', 'timestamp'));
		if (!empty($entries)) {
			foreach ($entries as &$entry) $entry = array_merge($entry, compact('path_id', 'timestamp'));
			$db->inject($gCfg['storage']['table_jobs'], $entries);
		}
		return $path_id;
	}


	static function resolve($path) {
		$aDb = DB::get(null, 'animebase');
		return $aDb->val($GLOBALS['gCfg']['storage']['table_base'], 'movie_id', compact('path'));
	}


	static function process($video_id, $shot, $flv, $postproc = true) {
		global $gCfg;
		$aDb = DB::get(null, 'animebase');
		$timestamp = Time::now();
		if ($postproc) $id = $aDb->rep($gCfg['storage']['table_postproc'], compact('video_id', 'shot', 'flv', 'timestamp'));
		else self::post($video_id, $shot, $flv, $timestamp);
			// DEPRECATED, UNUSED, OBSOLETE, NOT TESTED, DANGEROUS
			// should also add some metadata about added file here, possibly based on path
			//~ $id = $aDb->ins($gCfg['storage']['table_base'].TS.$type, compact('type', 'timestamp'));
			//~ $new_path = $gCfg['storage']['dir_usr'].DS.$type.DS.$id.'.'.String::crop(basename($path), '.');
			//~ if (!glob($new_path)) mkdir($new_path, 0755, true);
			//~ rename($path, $new_path);
		return $id;
	}


	static function finalize($id = null) {
		global $gCfg;
		$aDb = DB::get(null, 'animebase');
		$timestamp = Time::now();
		if ($id and is_array($id)) {
			$entries = $aDb->arr($gCfg['storage']['table_postproc'], 'video_id, shot, flv, timestamp', compact('id'));
			foreach ($entries as $entry) self::post($entry['video_id'], $entry['shot'], $entry['flv'], $entry['timestamp']);
		} else {
			while ($entry = $aDb->line('SELECT * FROM '.$gCfg['storage']['table_postproc'].' WHERE NOT ISNULL(video_id) AND processed = 0 LIMIT 1')) { // WHERE should be removed when and _if_ post-comparison will be implemented
				self::post($entry['video_id'], $entry['shot'], $entry['flv'], $entry['timestamp']);
				$aDb->upd($gCfg['storage']['table_postproc'], array('processed' => 1), $entry);
			}
		}
	}


	static function post($video_id, $shot, $flv, $timestamp = null) { // Possibly add drop_previous_flv/shots flag to input parameters
		global $gCfg;
		$aDb = DB::get(null, 'animebase');

		$data_add = !is_null($timestamp) ? $timestamp : Time::now();
		$title = str_replace('_', ' ', String::base(basename($shot), '.'));
		$type_id = $gCfg['types']['out_shot_type_id'];
		$movie_id = $video_id;
		$actual = 1;

		if (preg_match('/\b\d+\b/', $title, $num)) $sort = (int) reset($num);
		else $sort = $aDb->val($gCfg['storage']['table_shot'], 'MAX(sort)', compact('movie_id')) + 1;

		$shot_id = $aDb->ins($gCfg['storage']['table_shot'], compact('movie_id', 'type_id', 'data_add', 'actual', 'title', 'sort'));
		$flv_id = $aDb->ins('flv_list', compact('video_id', 'shot_id', 'path'));

		DH_ConvFS::post($shot_id, $flv_id, $shot, $flv, $type_id);
	}


	static function flush($path) {
		global $gCfg;
		$db = DB::get();
		$id = self::path($path);
		$db->del($gCfg['storage']['table_paths'], compact('id'));
		$db->del($gCfg['storage']['table_jobs'], array('path_id' => $id));
	}


	static function fetch($path_id, $status_mask = null, $cache = false) {
		global $gCfg;
		static $jobs_cache, $finished;
		$db = DB::get();
		$status_mask = '(status = 0)'.($status_mask ? ' OR (status & '.$db->num($status_mask).')' : '');
		if ($cache) {
			if (!$jobs_cache or $finished) {
				$jobs_cache = $db->arr($gCfg['storage']['table_jobs'], true, '*', $status_mask);
				$finished = false;
				return current($jobs_cache);
			}
			if ( ($job = next($jobs_cache)) === false ) $finished = true;
		} else $job = $db->line($gCfg['storage']['table_jobs'], true, '*', $status_mask);
		return $job;
	}


	static function status($id, $status) {
		global $gCfg;
		$db = DB::get();
		$timestamp = Time::now();
		return $db->upd($gCfg['storage']['table_jobs'], compact('status', 'timestamp'), compact('id'));
	}


	static function log($job_id, $message) {
		global $gCfg;
		$db = DB::get();
		$timestamp = Time::now();
		return $db->ins($gCfg['storage']['table_log'], compact('job_id', 'message', 'timestamp'));
	}


}
